% This code generates Figure 8 in "The Limited Power of Monetary Policy in
% a Pandemic", Antoine Lepetit and Cristina Fuentes-Albero, November 2021.

%% Preparation

clear all
clc

mp_pandemic_nozlb;

global M_ options_ oo_;
Tsim = 200;
options_.periods = Tsim;
options_.simul.maxit = 200; 
options_.dynatol.f = 1e-10;
options_.dynatol.x = 1e-10;

maxlag=1;
maxlead=1;
endo_simul = repmat(oo_.endo_simul(:,1),[1,Tsim+maxlag+maxlead]);
endo_simul(:,end) = oo_.endo_simul(:,end);
oo_.endo_simul = endo_simul;

% Calibrate shocks
mu = [0;zeros(Tsim+1-1,1)];
mp_shock = [0;zeros(Tsim+1-1,1)];
pi_s1_shock = [0;zeros(Tsim+1-1,1)];
pi_s2_shock = [0;0;0;0;0;-0.02;-0.04;-0.06;-0.08*ones(Tsim+1-8,1)];
pi_s3_shock = [0;-0.19;-0.19;-0.195;-0.195;-0.2025*ones(5,1);-0.205*ones(5,1);-0.2075*ones(10,1);-0.21*ones(Tsim+1-25,1)];

shocks = [mu,mp_shock,pi_s1_shock,pi_s2_shock,pi_s3_shock];

eps_t = shocks(1:2,:);
Tshock = size(eps_t,1);
nshock = size(eps_t,2);

%% Simulate baseline

Xn_baseline = endo_simul;

for h=1:Tshock
    
    display(sprintf('Now on time %d', h));
    
    options_.periods = Tsim-h+1;
    
    oo_.exo_simul = [zeros(1,nshock);eps_t(h,:);[zeros(Tsim-h+1,1),zeros(Tsim-h+1,1),zeros(Tsim-h+1,1),pi_s2_shock(h+1:end),pi_s3_shock(h+1:end)]];
    perfect_foresight_solver;
    
    Xn_baseline(:,h+1:end) = real(oo_.endo_simul(:,2:end));
    oo_.endo_simul = real(oo_.endo_simul(:,2:end));
    
end

clear oo_.endo_simul oo_.exo_simul mp_shock shocks eps_t Tshock nshock;

%% Simulate alt-sims with unanticipated monetary policy shocks happening in different periods

Xn = zeros(26,202,100);

  % Time 10

  j=10;
    
  mp_shock = [zeros(j,1);-0.0001;zeros(Tsim+1-(j+1),1)];

  shocks = [mu,mp_shock,pi_s1_shock,pi_s2_shock,pi_s3_shock]; 

  eps_t = shocks(1:j+1,:);
  Tshock = size(eps_t,1);
  nshock = size(eps_t,2);

  oo_.endo_simul = endo_simul;
  Xn(:,:,j) = endo_simul;

  for h=1:Tshock
    
      display(sprintf('Now on time %d', h));
    
      options_.periods = Tsim-h+1;
    
      oo_.exo_simul = [zeros(1,nshock);eps_t(h,:);[zeros(Tsim-h+1,1),zeros(Tsim-h+1,1),zeros(Tsim-h+1,1),pi_s2_shock(h+1:end),pi_s3_shock(h+1:end)]];
      perfect_foresight_solver;
    
      Xn(:,h+1:end,j) = real(oo_.endo_simul(:,2:end));
      oo_.endo_simul = real(oo_.endo_simul(:,2:end));
    
  end
  
  clear oo_.endo_simul oo_.exo_simul mp_shock shocks eps_t Tshock nshock;
  
  % Time 30

  j=30;
    
  mp_shock = [zeros(j,1);-0.0001;zeros(Tsim+1-(j+1),1)];

  shocks = [mu,mp_shock,pi_s1_shock,pi_s2_shock,pi_s3_shock]; 

  eps_t = shocks(1:j+1,:);
  Tshock = size(eps_t,1);
  nshock = size(eps_t,2);

  oo_.endo_simul = endo_simul;
  Xn(:,:,j) = endo_simul;

  for h=1:Tshock
    
      display(sprintf('Now on time %d', h));
    
      options_.periods = Tsim-h+1;
    
      oo_.exo_simul = [zeros(1,nshock);eps_t(h,:);[zeros(Tsim-h+1,1),zeros(Tsim-h+1,1),zeros(Tsim-h+1,1),pi_s2_shock(h+1:end),pi_s3_shock(h+1:end)]];
      perfect_foresight_solver;
    
      Xn(:,h+1:end,j) = real(oo_.endo_simul(:,2:end));
      oo_.endo_simul = real(oo_.endo_simul(:,2:end));
    
  end
  
  clear oo_.endo_simul oo_.exo_simul mp_shock shocks eps_t Tshock nshock;
  
  % Time 50

  j=50;
    
  mp_shock = [zeros(j,1);-0.0001;zeros(Tsim+1-(j+1),1)];

  shocks = [mu,mp_shock,pi_s1_shock,pi_s2_shock,pi_s3_shock]; 

  eps_t = shocks(1:j+1,:);
  Tshock = size(eps_t,1);
  nshock = size(eps_t,2);

  oo_.endo_simul = endo_simul;
  Xn(:,:,j) = endo_simul;

  for h=1:Tshock
    
      display(sprintf('Now on time %d', h));
    
      options_.periods = Tsim-h+1;
    
      oo_.exo_simul = [zeros(1,nshock);eps_t(h,:);[zeros(Tsim-h+1,1),zeros(Tsim-h+1,1),zeros(Tsim-h+1,1),pi_s2_shock(h+1:end),pi_s3_shock(h+1:end)]];
      perfect_foresight_solver;
    
      Xn(:,h+1:end,j) = real(oo_.endo_simul(:,2:end));
      oo_.endo_simul = real(oo_.endo_simul(:,2:end));
    
  end
  
  clear oo_.endo_simul oo_.exo_simul mp_shock shocks eps_t Tshock nshock;
  
  % Time 100

  j=100;
    
  mp_shock = [zeros(j,1);-0.0001;zeros(Tsim+1-(j+1),1)];

  shocks = [mu,mp_shock,pi_s1_shock,pi_s2_shock,pi_s3_shock]; 

  eps_t = shocks(1:j+1,:);
  Tshock = size(eps_t,1);
  nshock = size(eps_t,2);

  oo_.endo_simul = endo_simul;
  Xn(:,:,j) = endo_simul;

  for h=1:Tshock
    
      display(sprintf('Now on time %d', h));
    
      options_.periods = Tsim-h+1;
    
      oo_.exo_simul = [zeros(1,nshock);eps_t(h,:);[zeros(Tsim-h+1,1),zeros(Tsim-h+1,1),zeros(Tsim-h+1,1),pi_s2_shock(h+1:end),pi_s3_shock(h+1:end)]];
      perfect_foresight_solver;
    
      Xn(:,h+1:end,j) = real(oo_.endo_simul(:,2:end));
      oo_.endo_simul = real(oo_.endo_simul(:,2:end));
    
  end
  
  clear oo_.endo_simul oo_.exo_simul mp_shock shocks eps_t Tshock nshock;
  
  %% Figures

  plot(100.*(Xn(12,12:112,10)-Xn_baseline(12,12:112))./Xn_baseline(12,12:112),'Linewidth',3)
  hold on
  plot(100.*(Xn(12,32:132,30)-Xn_baseline(12,32:132))./Xn_baseline(12,32:132),'--','Linewidth',3)
  hold on
  plot(100.*(Xn(12,52:152,50)-Xn_baseline(12,52:152))./Xn_baseline(12,52:152),'-.','Linewidth',3)
  hold on
  plot(100.*(Xn(12,102:202,100)-Xn_baseline(12,100:200))./Xn_baseline(12,102:202),':','Linewidth',3)
  xlabel('Weeks after the shock','Fontsize',18)
  ylabel('Output (% deviation)','Fontsize',18)
  xlim([0 100])
  legend('Shock at time 10','Shock at time 30','Shock at time 50','Shock at time 100 - "normal times"','Fontsize',12)
